var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { AbortOperation, AbstractRemote } from '@journeyapps/powersync-sdk-common';
export class WebRemote extends AbstractRemote {
    post(path, data, headers = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const request = yield this.buildRequest(path);
            const res = yield fetch(request.url, {
                method: 'POST',
                headers: Object.assign(Object.assign({}, headers), request.headers),
                body: JSON.stringify(data)
            });
            if (!res.ok) {
                throw new Error(`Received ${res.status} - ${res.statusText} when posting to ${path}: ${yield res.text()}}`);
            }
            return res.json();
        });
    }
    get(path, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            const request = yield this.buildRequest(path);
            const res = yield fetch(request.url, {
                method: 'GET',
                headers: Object.assign(Object.assign({}, headers), request.headers)
            });
            if (!res.ok) {
                throw new Error(`Received ${res.status} - ${res.statusText} when getting from ${path}: ${yield res.text()}}`);
            }
            return res.json();
        });
    }
    postStreaming(path, data, headers = {}, signal) {
        return __awaiter(this, void 0, void 0, function* () {
            const request = yield this.buildRequest(path);
            /**
             * This abort controller will abort pending fetch requests.
             * If the request has resolved, it will be used to close the readable stream.
             * Which will cancel the network request.
             *
             * This nested controller is required since:
             *  Aborting the active fetch request while it is being consumed seems to throw
             *  an unhandled exception on the window level.
             */
            const controller = new AbortController();
            let requestResolved = false;
            signal === null || signal === void 0 ? void 0 : signal.addEventListener('abort', () => {
                var _a;
                if (!requestResolved) {
                    // Only abort via the abort controller if the request has not resolved yet
                    controller.abort((_a = signal.reason) !== null && _a !== void 0 ? _a : new AbortOperation('Cancelling network request before it resolves. Abort signal has been received.'));
                }
            });
            const res = yield fetch(request.url, {
                method: 'POST',
                headers: Object.assign(Object.assign({}, headers), request.headers),
                body: JSON.stringify(data),
                signal: controller.signal,
                cache: 'no-store'
            }).catch((ex) => {
                if (ex.name == 'AbortError') {
                    throw new AbortOperation(`Pending fetch request to ${request.url} has been aborted.`);
                }
                throw ex;
            });
            if (!res) {
                throw new Error('Fetch request was aborted');
            }
            requestResolved = true;
            if (!res.ok || !res.body) {
                const text = yield res.text();
                this.logger.error(`Could not POST streaming to ${path} - ${res.status} - ${res.statusText}: ${text}`);
                const error = new Error(`HTTP ${res.statusText}: ${text}`);
                error.status = res.status;
                throw error;
            }
            /**
             * The can-ndjson-stream does not handle aborted streams well.
             * This will intercept the readable stream and close the stream if
             * aborted.
             */
            const reader = res.body.getReader();
            // This will close the network request and read stream
            const closeReader = () => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield reader.cancel();
                }
                catch (ex) {
                    // an error will throw if the reader hasn't been used yet
                }
                reader.releaseLock();
            });
            signal === null || signal === void 0 ? void 0 : signal.addEventListener('abort', () => {
                closeReader();
            });
            const outputStream = new ReadableStream({
                start: (controller) => {
                    const processStream = () => __awaiter(this, void 0, void 0, function* () {
                        while (!(signal === null || signal === void 0 ? void 0 : signal.aborted)) {
                            try {
                                const { done, value } = yield reader.read();
                                // When no more data needs to be consumed, close the stream
                                if (done) {
                                    break;
                                }
                                // Enqueue the next data chunk into our target stream
                                controller.enqueue(value);
                            }
                            catch (ex) {
                                this.logger.error('Caught exception when reading sync stream', ex);
                                break;
                            }
                        }
                        if (!(signal === null || signal === void 0 ? void 0 : signal.aborted)) {
                            // Close the downstream readable stream
                            yield closeReader();
                        }
                        controller.close();
                    });
                    processStream();
                }
            });
            // Create a new response out of the intercepted stream
            return new Response(outputStream).body;
        });
    }
}
//# sourceMappingURL=WebRemote.js.map